Class: ZendeskAppsSupport::Package
- Inherits:
-
Object
- Object
- ZendeskAppsSupport::Package
- Extended by:
- Gem::Deprecate
- Includes:
- BuildTranslation
- Defined in:
- lib/zendesk_apps_support/package.rb
Constant Summary collapse
- MANIFEST_FILENAME =
'manifest.json'- REQUIREMENTS_FILENAME =
'requirements.json'- DEFAULT_LAYOUT =
Erubis::Eruby.new(File.read(File.('../assets/default_template.html.erb', __FILE__)))
- DEFAULT_SCSS =
File.read(File.('../assets/default_styles.scss', __FILE__))
- SRC_TEMPLATE =
Erubis::Eruby.new(File.read(File.('../assets/src.js.erb', __FILE__)))
- LOCATIONS_WITH_ICONS_PER_PRODUCT =
{ Product::SUPPORT => %w[top_bar nav_bar system_top_bar ticket_editor].freeze, Product::SELL => %w[top_bar].freeze }.freeze
Constants included from BuildTranslation
BuildTranslation::I18N_KEYS, BuildTranslation::I18N_TITLE_KEY, BuildTranslation::I18N_VALUE_KEY
Instance Attribute Summary collapse
-
#lib_root ⇒ Object
readonly
Returns the value of attribute lib_root.
-
#root ⇒ Object
readonly
Returns the value of attribute root.
-
#warnings ⇒ Object
readonly
Returns the value of attribute warnings.
Class Method Summary collapse
Instance Method Summary collapse
- #app_css ⇒ Object
- #app_js ⇒ Object
- #assets ⇒ Object
-
#compile(options) ⇒ Object
(also: #compile_js)
this is not really compile_js, it compiles the whole app including scss for v1 apps.
- #compiled_templates(app_id, asset_url_prefix) ⇒ Object
- #files ⇒ Object
- #has_file?(path) ⇒ Boolean
- #has_requirements? ⇒ Boolean
- #has_svgs? ⇒ Boolean
- #html_files ⇒ Object
- #iframe_only? ⇒ Boolean
-
#initialize(dir, is_cached = true) ⇒ Package
constructor
A new instance of Package.
- #is_no_template ⇒ Object
- #js_files ⇒ Object
- #lib_files ⇒ Object
- #locales ⇒ Object
- #location_icons ⇒ Object
- #manifest ⇒ Object
- #manifest_json ⇒ Object
- #no_template_locations ⇒ Object
- #path_to(file) ⇒ Object
- #requirements_json ⇒ Object
- #requirements_path ⇒ Object
- #svg_files ⇒ Object
- #template_files ⇒ Object
- #templates ⇒ Object
- #text_files ⇒ Object
- #translation_files ⇒ Object
- #translations ⇒ Object
- #translations_for(locale) ⇒ Object
- #validate(options = {}) ⇒ Object
- #validate!(opts = {}) ⇒ Object
Methods included from BuildTranslation
#remove_zendesk_keys, #to_flattened_namespaced_hash
Constructor Details
#initialize(dir, is_cached = true) ⇒ Package
Returns a new instance of Package.
26 27 28 29 30 31 32 |
# File 'lib/zendesk_apps_support/package.rb', line 26 def initialize(dir, is_cached = true) @root = Pathname.new(File.(dir)) @lib_root = Pathname.new(File.join(root, 'lib')) @is_cached = is_cached # disabled by ZAT for development @warnings = [] end |
Instance Attribute Details
#lib_root ⇒ Object (readonly)
Returns the value of attribute lib_root.
24 25 26 |
# File 'lib/zendesk_apps_support/package.rb', line 24 def lib_root @lib_root end |
#root ⇒ Object (readonly)
Returns the value of attribute root.
24 25 26 |
# File 'lib/zendesk_apps_support/package.rb', line 24 def root @root end |
#warnings ⇒ Object (readonly)
Returns the value of attribute warnings.
24 25 26 |
# File 'lib/zendesk_apps_support/package.rb', line 24 def warnings @warnings end |
Class Method Details
.has_custom_object_requirements?(requirements_hash) ⇒ Boolean
222 223 224 225 226 227 228 229 230 |
# File 'lib/zendesk_apps_support/package.rb', line 222 def self.has_custom_object_requirements?(requirements_hash) return false if requirements_hash.nil? custom_object_requirements = requirements_hash.fetch(AppRequirement::CUSTOM_OBJECTS_KEY, {}) types = custom_object_requirements.fetch(AppRequirement::CUSTOM_OBJECTS_TYPE_KEY, []) relationships = custom_object_requirements.fetch(AppRequirement::CUSTOM_OBJECTS_RELATIONSHIP_TYPE_KEY, []) (types | relationships).any? end |
Instance Method Details
#app_css ⇒ Object
232 233 234 235 236 |
# File 'lib/zendesk_apps_support/package.rb', line 232 def app_css return File.read(path_to('app.scss')) if has_file?('app.scss') return File.read(path_to('app.css')) if has_file?('app.css') '' end |
#app_js ⇒ Object
238 239 240 241 242 243 244 |
# File 'lib/zendesk_apps_support/package.rb', line 238 def app_js if @is_cached @app_js ||= read_file('app.js') else read_file('app.js') end end |
#assets ⇒ Object
74 75 76 77 78 |
# File 'lib/zendesk_apps_support/package.rb', line 74 def assets @assets ||= Dir.chdir(root) do Dir['assets/**/*'].select { |f| File.file?(f) } end end |
#compile(options) ⇒ Object Also known as: compile_js
this is not really compile_js, it compiles the whole app including scss for v1 apps
132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 |
# File 'lib/zendesk_apps_support/package.rb', line 132 def compile() begin app_id = .fetch(:app_id) asset_url_prefix = .fetch(:assets_dir) name = .fetch(:app_name) rescue KeyError => e raise ArgumentError, e. end locale = .fetch(:locale, 'en') source = manifest.iframe_only? ? nil : app_js app_class_name = "app-#{app_id}" # if no_template is an array, we still need the templates templates = manifest.no_template == true ? {} : compiled_templates(app_id, asset_url_prefix) SRC_TEMPLATE.result( name: name, version: manifest.version, source: source, app_class_properties: manifest.app_class_properties, asset_url_prefix: asset_url_prefix, logo_asset_hash: generate_logo_hash(manifest.products), location_icons: location_icons, app_class_name: app_class_name, author: manifest., translations: manifest.iframe_only? ? nil : runtime_translations(translations_for(locale)), framework_version: manifest.framework_version, templates: templates, modules: commonjs_modules, iframe_only: manifest.iframe_only? ) end |
#compiled_templates(app_id, asset_url_prefix) ⇒ Object
193 194 195 196 197 198 199 200 201 202 |
# File 'lib/zendesk_apps_support/package.rb', line 193 def compiled_templates(app_id, asset_url_prefix) compiler = ZendeskAppsSupport::StylesheetCompiler.new(DEFAULT_SCSS + app_css, app_id, asset_url_prefix) compiled_css = compiler.compile(sassc: manifest.enabled_experiments.include?('newCssCompiler')) layout = templates['layout'] || DEFAULT_LAYOUT.result templates.tap do |templates| templates['layout'] = "<style>\n#{compiled_css}</style>\n#{layout}" end end |
#files ⇒ Object
92 93 94 95 96 97 98 99 100 101 |
# File 'lib/zendesk_apps_support/package.rb', line 92 def files files = [] Dir[root.join('**/**')].each do |f| next unless File.file?(f) relative_file_name = f.sub(%r{#{root}/?}, '') next if relative_file_name =~ %r{^tmp/} files << AppFile.new(self, relative_file_name) end files end |
#has_file?(path) ⇒ Boolean
210 211 212 |
# File 'lib/zendesk_apps_support/package.rb', line 210 def has_file?(path) File.file?(path_to(path)) end |
#has_requirements? ⇒ Boolean
218 219 220 |
# File 'lib/zendesk_apps_support/package.rb', line 218 def has_requirements? has_file?(REQUIREMENTS_FILENAME) end |
#has_svgs? ⇒ Boolean
214 215 216 |
# File 'lib/zendesk_apps_support/package.rb', line 214 def has_svgs? svg_files.any? end |
#html_files ⇒ Object
111 112 113 |
# File 'lib/zendesk_apps_support/package.rb', line 111 def html_files @html_files ||= files.select { |f| f =~ %r{.*\.html?$} } end |
#iframe_only? ⇒ Boolean
246 247 248 |
# File 'lib/zendesk_apps_support/package.rb', line 246 def iframe_only? manifest.iframe_only? end |
#is_no_template ⇒ Object
183 184 185 |
# File 'lib/zendesk_apps_support/package.rb', line 183 def is_no_template manifest.no_template? end |
#js_files ⇒ Object
107 108 109 |
# File 'lib/zendesk_apps_support/package.rb', line 107 def js_files @js_files ||= files.select { |f| f =~ %r{^.*\.jsx?$} } end |
#lib_files ⇒ Object
115 116 117 |
# File 'lib/zendesk_apps_support/package.rb', line 115 def lib_files @lib_files ||= js_files.select { |f| f =~ %r{^lib/} } end |
#locales ⇒ Object
88 89 90 |
# File 'lib/zendesk_apps_support/package.rb', line 88 def locales translations.keys end |
#location_icons ⇒ Object
285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 |
# File 'lib/zendesk_apps_support/package.rb', line 285 def location_icons Hash.new { |h, k| h[k] = {} }.tap do |location_icons| manifest..each do || # no location information in the manifest next unless .location product = .location.product location_name = .location.name # the location on the product does not support icons next unless LOCATIONS_WITH_ICONS_PER_PRODUCT.fetch(product, []).include?(location_name) host = .location.product.name product_directory = manifest.products.count > 1 ? "#{host}/" : '' location_icons[host][location_name] = build_location_icons_hash(location_name, product_directory) end end end |
#manifest ⇒ Object
174 175 176 |
# File 'lib/zendesk_apps_support/package.rb', line 174 def manifest @manifest ||= Manifest.new(read_file(MANIFEST_FILENAME)) end |
#manifest_json ⇒ Object
169 170 171 |
# File 'lib/zendesk_apps_support/package.rb', line 169 def manifest_json @manifest_json ||= read_json(MANIFEST_FILENAME) end |
#no_template_locations ⇒ Object
188 189 190 |
# File 'lib/zendesk_apps_support/package.rb', line 188 def no_template_locations manifest.no_template_locations end |
#path_to(file) ⇒ Object
80 81 82 |
# File 'lib/zendesk_apps_support/package.rb', line 80 def path_to(file) File.join(root, file) end |
#requirements_json ⇒ Object
178 179 180 181 |
# File 'lib/zendesk_apps_support/package.rb', line 178 def requirements_json return nil unless has_requirements? @requirements ||= read_json(REQUIREMENTS_FILENAME, object_class: Manifest::NoOverrideHash) end |
#requirements_path ⇒ Object
84 85 86 |
# File 'lib/zendesk_apps_support/package.rb', line 84 def requirements_path path_to(REQUIREMENTS_FILENAME) end |
#svg_files ⇒ Object
119 120 121 |
# File 'lib/zendesk_apps_support/package.rb', line 119 def svg_files @svg_files ||= files.select { |f| f =~ %r{^assets/.*\.svg$} } end |
#template_files ⇒ Object
123 124 125 |
# File 'lib/zendesk_apps_support/package.rb', line 123 def template_files files.select { |f| f =~ %r{^templates/.*\.hdbs$} } end |
#templates ⇒ Object
251 252 253 254 255 256 257 258 259 |
# File 'lib/zendesk_apps_support/package.rb', line 251 def templates templates_dir = path_to('templates') Dir["#{templates_dir}/*.hdbs"].each_with_object({}) do |file, memo| str = File.read(file) str.chomp! memo[File.basename(file, File.extname(file))] = str memo end end |
#text_files ⇒ Object
103 104 105 |
# File 'lib/zendesk_apps_support/package.rb', line 103 def text_files @text_files ||= files.select { |f| f =~ %r{.*(html?|xml|js|json?)$} } end |
#translation_files ⇒ Object
127 128 129 |
# File 'lib/zendesk_apps_support/package.rb', line 127 def translation_files files.select { |f| f =~ %r{^translations/} } end |
#translations ⇒ Object
261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 |
# File 'lib/zendesk_apps_support/package.rb', line 261 def translations return @translations if @is_cached && @translations @translations = begin translation_dir = path_to('translations') return {} unless File.directory?(translation_dir) locale_path = "#{translation_dir}/#{manifest.default_locale}.json" default_translations = process_translations(locale_path, default_locale: true) Dir["#{translation_dir}/*.json"].each_with_object({}) do |path, memo| locale = File.basename(path, File.extname(path)) locale_translations = if locale == manifest.default_locale default_translations else deep_merge_hash(default_translations, process_translations(path)) end memo[locale] = locale_translations end end end |
#translations_for(locale) ⇒ Object
204 205 206 207 208 |
# File 'lib/zendesk_apps_support/package.rb', line 204 def translations_for(locale) trans = translations return trans[locale] if trans[locale] trans[manifest.default_locale] end |
#validate(options = {}) ⇒ Object
34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 |
# File 'lib/zendesk_apps_support/package.rb', line 34 def validate( = {}) marketplace = .fetch(:marketplace, true) skip_marketplace_translations = .fetch(:skip_marketplace_translations, false) errors = [] errors << Validations::Manifest.call(self) if has_valid_manifest?(errors) errors << Validations::Marketplace.call(self) if marketplace errors << Validations::Source.call(self) errors << Validations::Translations.call(self, skip_marketplace_translations: skip_marketplace_translations) errors << Validations::Requirements.call(self) # only adds warnings Validations::SecureSettings.call(self) Validations::Requests.call(self) unless manifest.requirements_only? || manifest.marketing_only? || manifest.iframe_only? errors << Validations::Templates.call(self) errors << Validations::Stylesheets.call(self) end end errors << Validations::Banner.call(self) if errors << Validations::Svg.call(self) if has_svgs? errors << Validations::Mime.call(self) # only adds warnings Validations::Secrets.call(self) errors.flatten.compact end |
#validate!(opts = {}) ⇒ Object
67 68 69 70 71 72 |
# File 'lib/zendesk_apps_support/package.rb', line 67 def validate!(opts = {}) errors = validate(opts) raise errors.first if errors.any? true end |